// CelsiusFahrenheitConverter.java
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.BorderFactory;

import net.sourceforge.jcomponentbreadboard.JComponentBreadboard;

@SuppressWarnings("serial")
public class CelsiusFahrenheitConverter extends JComponentBreadboard {
   final double DEFAULT_C = 25;  // room temperature
   final double MIN_C = -40;     // -40 C == -40 F
   final double MAX_C = 100.0;   // water's boiling point
   final double DELTA_C = 1;     // Celsius-spinner change per click        
   final double DELTA_F = 1;     // Fahrenheit-spinner change per click
   private double cToF(double c) { // Celsius to Fahrenheit
      return (9./5.)*c + 32.0; 
   } 
   private double fToC(double f) { // Fahrenheit to Celsius
      return (f-32)*(5./9.);   
   }   
   
   public CelsiusFahrenheitConverter() {
      JLabel  celsiusLbl = new JLabel("Degrees C: ");
      JSpinner celsiusSpinner = new JSpinner(new SpinnerNumberModel(
         DEFAULT_C, MIN_C, MAX_C, DELTA_C)); 
      JSlider celsiusSlider = new JSlider(
         (int) MIN_C, (int) MAX_C);

      JLabel  fahrenheitLbl = new JLabel("Degrees F: ");
      JSpinner fahrenheitSpinner = new JSpinner(new SpinnerNumberModel(
         cToF(DEFAULT_C), cToF(MIN_C), cToF(MAX_C), DELTA_F)); 
      JSlider fahrenheitSlider = new JSlider(
         (int) cToF(MIN_C), (int) cToF(MAX_C));
      
      setBreadboard(new Object[][] {
         {null,    SHRINKS,                    EXPANDS},	  
         {BISCALE, yAlign(0,celsiusSlider),    celsiusSlider},
         {NOSCALE, xAlign(1,celsiusLbl),       xFill(celsiusSpinner)},
         {NOSCALE, xAlign(1,fahrenheitLbl),    xFill(fahrenheitSpinner)},
         {BISCALE, yAlign(1,fahrenheitSlider), fahrenheitSlider},
      });
      setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

      jbConnect(celsiusSpinner, "degreesC");
      jbConnect(celsiusSlider, "intDegreesC");

      jbConnect(fahrenheitSpinner, "degreesF");
      jbConnect(fahrenheitSlider, "intDegreesF");

   }
   
   double degreesC = DEFAULT_C;  // what everything views and controls
 
   public double getDegreesC() { return degreesC; }    
   public void   setDegreesC(double c) { degreesC = c;}
   public int getIntDegreesC() {return (int) Math.rint(degreesC);}
   public void setIntDegreesC(int c) {degreesC = c;}

   public double getDegreesF() { return cToF(degreesC); }
   public void   setDegreesF(double f) { degreesC = fToC(f);}
   public int getIntDegreesF() {return (int) Math.rint(getDegreesF());}
   public void setIntDegreesF(int f) {setDegreesF(f);}


   public static void main(String[] args) {
      showMessageBreadboard(null,
                            new CelsiusFahrenheitConverter(),
                            "C to F and F to C");
  }
   
}
