// DateChooser.java
import java.awt.Color;
import java.awt.Container;
import java.util.Calendar;
import java.util.Date;
import java.util.Hashtable;

import javax.swing.*;
import net.sourceforge.jcomponentbreadboard.JComponentBreadboard;

@SuppressWarnings("serial")
public class DateChooser extends JComponentBreadboard {
   private final int MIN_YEAR = 1900;
   private final int MAX_YEAR = 2099;
   private Calendar calendar = Calendar.getInstance();
   private String[] weekDayLabels = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
   private final int DAYS_PER_WEEK = weekDayLabels.length;
   private String[] listOfMonths = {
      "January", "February", "March", "April", "May", "June",
      "July", "August", "September", "October", "November", "December"};
   // each month remembers the last selected day; initialize with day 1
   int[] selectedDay = {1,1,1,1,1,1,1,1,1,1,1,1}; 
   int selectedMonth;
   int selectedYear;
   Date initialDate;
   JComponent[][] dayOfMonth;
   
   public DateChooser(Date initialDate) {
      this.initialDate = initialDate;       
            
      calendar.setTime((null == initialDate) ? new Date() : initialDate);
      selectedYear = calendar.get(Calendar.YEAR);
      selectedMonth = calendar.get(Calendar.MONTH);
      selectedDay[selectedMonth]   = calendar.get(Calendar.DAY_OF_MONTH);

      // raise exception for dates beyond range we can handle
      if (selectedYear < MIN_YEAR || selectedYear > MAX_YEAR)
         throw new IllegalArgumentException("Initial date's year, " + selectedYear +
                                            " is not in the required range: " +
                                            MIN_YEAR + "..." + MAX_YEAR);
      
      JSpinner yearSpinner = new JSpinner(new SpinnerListModel(intList(MIN_YEAR, MAX_YEAR)));
      JComboBox monthComboBox = new JComboBox(listOfMonths);
      monthComboBox.setMaximumRowCount(listOfMonths.length);
      JSpinner daySpinner = new JSpinner(new SpinnerListModel(getDaySpinnerItems()));

      JComponent monthSlider = yUpsize(10, new JSlider());
      
      JLabel dateLabel = new JLabel();
      
      // dropdown lists, spinners, labels for selecting/displaying year, month, and day
      JComponent yearMonthDayBar = xFill(yUpsize(10, new JComponentBreadboard(
         new Object[][] {
           {null,    NOSCALE,                 NOSCALE,    NOSCALE,                        NOSCALE,    NOSCALE,                       BISCALE,    NOSCALE},
           {NOSCALE, xAlign(0,monthComboBox), xSpace(10), yFill(xAlign(0.5,daySpinner)),  xSpace(10), yFill(xAlign(1, yearSpinner)), xSpace(10), dateLabel}
         }
      )));

      JButton ok = new JButton("Choose Date");
      JButton cancel = new JButton("Cancel");

      JComponent commandButtonBar = xAlign(1, new JComponentBreadboard(new Object[][] {
         {null,    BISCALE,    NOSCALE,   NOSCALE,    NOSCALE },
         {NOSCALE, ySpace(10), null,      null,       null  }, 
         {NOSCALE, xSpace(10), ok,        xSpace(10), cancel}
      }));
      
      JComponent[] dayOfWeekHeader = new JLabel[DAYS_PER_WEEK];
// why it's 6: partial week on top + 4 full weeks in middle + partial on bottom 
      final int MAX_DISPLAYED_WEEKS = 6;
      dayOfMonth = new JToggleButton[MAX_DISPLAYED_WEEKS][DAYS_PER_WEEK];
      for (int i = 0; i < DAYS_PER_WEEK; i++) {
         dayOfWeekHeader[i] = xAlign(0.5, new JLabel(weekDayLabels[i]));
         for (int j = 0; j < MAX_DISPLAYED_WEEKS; j++) {
            dayOfMonth[j][i] = xyFill(new JToggleButton());
         }
      }
      
      setBreadboard(new Object[][] {
         {null,    EXPANDS,          __,     __,     __,     __,     __,    __},
         {NOSCALE, yearMonthDayBar,  __,     __,     __,     __,     __,    __},
         {"",      monthSlider,      __,     __,     __,     __,     __,    __},        
         {"",      dayOfWeekHeader,  __,     __,     __,     __,     __,    __},
         {EXPANDS, dayOfMonth,       __,     __,     __,     __,     __,    __},          
         {"",      "",               __,     __,     __,     __,     __,    __},          
         {"",      "",               __,     __,     __,     __,     __,    __},          
         {"",      "",               __,     __,     __,     __,     __,    __},          
         {"",      "",               __,     __,     __,     __,     __,    __},          
         {"",      "",               __,     __,     __,     __,     __,    __},
         {NOSCALE, commandButtonBar, __,     __,     __,     __,     __,    __}
      });
      setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
      jbConnect(daySpinner, "daySpinner");
      jbConnect(monthComboBox, "month");
      jbConnect(monthSlider, "monthSlider");
      jbConnect(yearSpinner, "year");
      jbConnect(dayOfMonth, "dayOfMonth");
      jbConnect(ok, "ok");
      jbConnect(CLOSE_BUTTON, "ok");
      jbConnect(cancel, "cancel");
      jbConnect(KeyStroke.getKeyStroke("ESCAPE"), "cancel");
      jbConnect(dateLabel, "dateLabel");
      jbConnect(KeyStroke.getKeyStroke("LEFT"), "left");
      jbConnect(KeyStroke.getKeyStroke("RIGHT"), "right");
      jbConnect(KeyStroke.getKeyStroke("UP"),    "up");
      jbConnect(KeyStroke.getKeyStroke("DOWN"),  "down");
   }
   
   // returns integer sequence from iMin to iMax as an array of Strings
   private String[] intList(int iMin, int iMax) {
      String[] result = new String[iMax - iMin + 1];
      for (int i = 0; i < iMax-iMin+1; i++)
         result[i] = Integer.toString(iMin + i);
      return result;
   }  
   // move the calendar to the date associated with the specified row and column
   private void gotoDate(int iRow, int iCol) {
      calendar.set(selectedYear, selectedMonth, 1);
      // value returned for day-of-week is 1-Sun, 2-Mon, but we want 0-Sun, 1-Mon
      int firstDay = calendar.get(Calendar.DAY_OF_WEEK)-1;
      calendar.add(Calendar.DAY_OF_MONTH, iRow * DAYS_PER_WEEK + iCol - firstDay);
   }
   // togglebutton is only selected (in "pressed state") if it
   // represents the currently selected date
   public boolean getDayOfMonth(int iRow, int iCol) {
      gotoDate(iRow, iCol);
      boolean result = calendar.get(Calendar.YEAR) == selectedYear &&
                       calendar.get(Calendar.MONTH) == selectedMonth &&
                       calendar.get(Calendar.DAY_OF_MONTH) == selectedDay[selectedMonth];
      return result;
   }
   public void setDayOfMonth(int iRow, int iCol, boolean value) {   
      gotoDate(iRow, iCol);
      selectedYear = calendar.get(Calendar.YEAR);
      selectedMonth = calendar.get(Calendar.MONTH);
      selectedDay[selectedMonth] = calendar.get(Calendar.DAY_OF_MONTH);
   }
   public String getDayOfMonthText(int iRow, int iCol) {
      gotoDate(iRow, iCol);
      String result = calendar.get(Calendar.DAY_OF_MONTH) + ""; 
      return result;
   }
   public boolean getDayOfMonthContentAreaFilled(int iRow, int iCol) {
      boolean result = getDayOfMonth(iRow, iCol);
      return result;
   }                    
   public Color getDayOfMonthForeground(int iRow, int iCol) {
      Color result = getDayOfMonth(iRow, iCol) ? Color.RED : Color.BLACK;
      return result;
   }
   // Disables "before month" cells at top and "after month" cells at bottom   
   public boolean getDayOfMonthEnabled(int iRow, int iCol) {
      gotoDate(iRow, iCol);
      boolean result = (calendar.get(Calendar.MONTH) == selectedMonth);
      return result;
   }
   
   
// these next 4 methods are connected to same-named keystrokes to allow
// navigation of the monthly calendar region via the arrow keys 
   public void setLeft() {
      selectedDay[selectedMonth] = Math.max(selectedDay[selectedMonth]-1, 1);
   }
   public void setRight() {	   
      selectedDay[selectedMonth] = Math.min(selectedDay[selectedMonth]+1,
                                     calendar.getActualMaximum(Calendar.DAY_OF_MONTH));
   }
   public void setUp() {
      if (selectedDay[selectedMonth] - DAYS_PER_WEEK >= 1) {
         selectedDay[selectedMonth] = selectedDay[selectedMonth] - DAYS_PER_WEEK;
      }
   }
   public void setDown() {	   
      if (selectedDay[selectedMonth] + DAYS_PER_WEEK <=
          calendar.getActualMaximum(Calendar.DAY_OF_MONTH)) {
         selectedDay[selectedMonth] = selectedDay[selectedMonth] + DAYS_PER_WEEK;
      }  
   }

   public void setOk() {
     calendar.set(selectedYear, selectedMonth, selectedDay[selectedMonth]); 
     jbReturn(calendar.getTime());
   }
   public boolean getOkIsDefaultButton() { return true; }

   public void setCancel() {
      jbReturn(initialDate);
   }

   public String getMonth() {
     return listOfMonths[selectedMonth]; 
   }

   // assures we don't exceed max day setting during leap years (or similar oddities)
   private void patchForLeapYears() {
      calendar.set(selectedYear, selectedMonth, 1);
      selectedDay[selectedMonth] = Math.min(selectedDay[selectedMonth],
                                            calendar.getActualMaximum(Calendar.DAY_OF_MONTH));
   }
   
   public void setMonth(String value) {
      for (int i=0; i < listOfMonths.length; i++) {
         if (value.equals(listOfMonths[i])) {
            selectedMonth = i;
            break;
         }
      }
      patchForLeapYears();      
   }

   public String getYear() {
     return selectedYear + "";
   }
   public void setYear(String value) {
      selectedYear = Integer.parseInt(value);
      patchForLeapYears();
   }   
   
   public String getDateLabelText() {
      return listOfMonths[selectedMonth] + " " + selectedDay[selectedMonth] + ", " + selectedYear;
   }

   public String getDaySpinner() {
      return selectedDay[selectedMonth] + "";
   }
   public void setDaySpinner(String value) {
      selectedDay[selectedMonth] = Integer.parseInt(value);
   }
   
   public String[] getDaySpinnerItems() {
      calendar.set(selectedYear, selectedMonth, selectedDay[selectedMonth]);
      String[] result = intList(1, calendar.getActualMaximum(Calendar.DAY_OF_MONTH));
      return result;
   }

   public int getMonthSlider() {
      return selectedMonth;
   }
   public void setMonthSlider(int value) {
      selectedMonth = value;
   }
   public Hashtable<Integer, JLabel> getMonthSliderLabelTable() {
      Hashtable<Integer, JLabel> result = new Hashtable<Integer, JLabel>();
      for (int i = 1; i <= listOfMonths.length; i++)
         result.put(i-1, new JLabel(i + ""));
      return result;
   }
   public int getMonthSliderMajorTickSpacing() {return 1;}
   public int getMonthSliderMaximum() {return listOfMonths.length-1;}
   public int getMonthSliderMinimum() {return 0;}
   public boolean getMonthSliderPaintLabels() {return true;}
   public boolean getMonthSliderPaintTicks() {return false;}
   public boolean getMonthSliderPaintTrack() {return true;}
   
   // The static method clients will use to invoke the modal DateChooser
   // to get a date from the user. Everything else exists to implement this.
   public static Date showDateChooser(Container parent, Date initialDate, String title) {
      Date result = (Date) showInputBreadboard(parent, new DateChooser(initialDate), title);
      return result;
   }

   // tests the date-chooser popup modal dialog
   public static void main(String[] args) {
      Date result = showDateChooser(null, new Date(), "Choose a Date");
      JOptionPane.showMessageDialog(null, "The chooser returned: " + result);
   }
   
}
