// FirefoxClearPrivateDataForm.java
import java.awt.event.KeyEvent;

import javax.swing.*;
import net.sourceforge.jcomponentbreadboard.JComponentBreadboard;

// This class is our stand-in for that part of Firefox responsible for
// managing/maintaining private user data (that the form provides a GUI for clearing).
class FirefoxPrivateData {
// Internal state representation flags: define if there is anything that needs clearing   
   public boolean isBrowsingHistoryCleared = false;
   public boolean isDownloadHistoryCleared = false;
   public boolean isSearchHistoryCleared = false;
   public boolean isCacheCleared = false;
   public boolean areCookiesCleared = false;
   public boolean areSavedPasswordsCleared = false;
   public boolean areAuthenticatedSessionsCleared = false;

// User preference flags: define what data gets cleared when user clicks the
// "Clear Private Data Now" button  
   public boolean clearBrowsingHistory = true;
   public boolean clearDownloadHistory = true;
   public boolean clearSearchHistory = true;
   public boolean clearCache = true;
   public boolean clearCookies = false;
   public boolean clearSavedPasswords = false;
   public boolean clearAuthenticatedSessions = true;

// If this were Firefox, this would actually clear out the data (not just zap flags)   
   public void clearPrivateData() {
      if (clearBrowsingHistory) isBrowsingHistoryCleared = true;
      if (clearDownloadHistory) isDownloadHistoryCleared = true;
      if (clearSearchHistory) isSearchHistoryCleared = true;
      if (clearCache) isCacheCleared = true;
      if (clearCookies) areCookiesCleared = true;
      if (clearSavedPasswords) areSavedPasswordsCleared = true;
      if (clearAuthenticatedSessions) areAuthenticatedSessionsCleared = true;
   }
}

@SuppressWarnings("serial")
public class FirefoxClearPrivateDataForm extends JComponentBreadboard {

   private FirefoxPrivateData privateData;
   
   public FirefoxClearPrivateDataForm(FirefoxPrivateData privateData) {
      this.privateData = privateData;
      JCheckBox browsingHistory = new JCheckBox("Browsing History");
      browsingHistory.setMnemonic(KeyEvent.VK_B);
      JCheckBox downloadHistory = new JCheckBox("Download History");
      downloadHistory.setMnemonic(KeyEvent.VK_D);
      JCheckBox searchHistory = new JCheckBox("Saved Form and Search History");
      searchHistory.setMnemonic(KeyEvent.VK_F);
      JCheckBox cache = new JCheckBox("Cache");
      cache.setMnemonic(KeyEvent.VK_A);
      JCheckBox cookies = new JCheckBox("Cookies");
      cookies.setMnemonic(KeyEvent.VK_C);
      JCheckBox savedPasswords = new JCheckBox("Saved Passwords");
      savedPasswords.setMnemonic(KeyEvent.VK_P);
      JCheckBox authenticatedSessions = new JCheckBox("Authenticated Sessions");
      authenticatedSessions.setMnemonic(KeyEvent.VK_S);
      JButton clearPrivateDataNow = new JButton("Clear Private Data Now");
      JButton cancel = new JButton("Cancel");
      JComponent buttonBar = new JComponentBreadboard(
         new Object[][] {{xSpace(200),clearPrivateDataNow, xSpace(10), cancel}});
      JLabel instructions = new JLabel("Clear the following items now:");
      final int SPACE = 5;  // adds extra vertical space between checkboxes
      setBreadboard(new Object[][] {
         {instructions},
         {ySpace(SPACE)},
         {browsingHistory},
         {ySpace(SPACE)},
         {downloadHistory},
         {ySpace(SPACE)},
         {searchHistory},
         {ySpace(SPACE)},
         {cache},
         {ySpace(SPACE)},
         {cookies},
         {ySpace(SPACE)},
         {savedPasswords},
         {ySpace(SPACE)},
         {authenticatedSessions},
         {ySpace(SPACE)},
         {buttonBar},
         {ySpace(SPACE)},
      });
      setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
      
      jbConnect(browsingHistory, "browsingHistory");
      jbConnect(downloadHistory, "downloadHistory");
      jbConnect(searchHistory, "searchHistory");
      jbConnect(cache, "cache");
      jbConnect(cookies, "cookies");
      jbConnect(savedPasswords, "savedPasswords");
      jbConnect(authenticatedSessions, "authenticatedSessions");
      jbConnect(clearPrivateDataNow, "clearPrivateDataNow");
      jbConnect(cancel, "cancel");
      jbConnect(CLOSE_BUTTON, "cancel");
      
   }
   // our screen shot grabber needs a no-arg constructor
   public FirefoxClearPrivateDataForm() {
      this(new FirefoxPrivateData());
   }
   
   public boolean getBrowsingHistory() {
      return privateData.clearBrowsingHistory;       
   }
   public void setBrowsingHistory(boolean value) {
      privateData.clearBrowsingHistory = value;
   }
   public boolean getBrowsingHistoryEnabled() {
     return !privateData.isBrowsingHistoryCleared; 
   }

   public boolean getDownloadHistory() {
      return privateData.clearDownloadHistory;       
   }
   public void setDownloadHistory(boolean value) {
      privateData.clearDownloadHistory = value;
   }
   public boolean getDownloadHistoryEnabled() {
     return !privateData.isDownloadHistoryCleared; 
   }
   
   public boolean getSearchHistory() {
      return privateData.clearSearchHistory;       
   }
   public void setSearchHistory(boolean value) {
      privateData.clearSearchHistory = value;
   }
   public boolean getSearchHistoryEnabled() {
     return !privateData.isSearchHistoryCleared; 
   }
   
   public boolean getCache() {
      return privateData.clearCache;       
   }
   public void setCache(boolean value) {
      privateData.clearCache = value;
   }
   public boolean getCacheEnabled() {
     return !privateData.isCacheCleared; 
   }
   
   public boolean getCookies() {
      return privateData.clearCookies;       
   }
   public void setCookies(boolean value) {
      privateData.clearCookies = value;
   }
   public boolean getCookiesEnabled() {
     return !privateData.areCookiesCleared; 
   }
   
   public boolean getSavedPasswords() {
      return privateData.clearSavedPasswords;       
   }
   public void setSavedPasswords(boolean value) {
      privateData.clearSavedPasswords = value;
   }
   public boolean getSavedPasswordsEnabled() {
     return !privateData.areSavedPasswordsCleared; 
   }
   
   public boolean getAuthenticatedSessions() {
      return privateData.clearAuthenticatedSessions;       
   }
   public void setAuthenticatedSessions(boolean value) {
      privateData.clearAuthenticatedSessions = value;
   }
   public boolean getAuthenticatedSessionsEnabled() {
     return !privateData.areAuthenticatedSessionsCleared; 
   }

   public void setClearPrivateDataNow() {
      privateData.clearPrivateData();
	   jbReturn(new Boolean(true));
   }
   public boolean getClearPrivateDataNowEnabled() {
      return (getBrowsingHistory() && getBrowsingHistoryEnabled()) ||
            (getDownloadHistory() && getDownloadHistoryEnabled()) ||
            (getSearchHistory() && getSearchHistoryEnabled()) ||
            (getCache() && getCacheEnabled()) ||
            (getCookies() && getCookiesEnabled() )||
            (getSavedPasswords() && getSavedPasswordsEnabled()) ||
            (getAuthenticatedSessions() && getAuthenticatedSessionsEnabled());
   }
   public void setCancel() {
//Cancel should undo changes, but since our app ends anyway, it doesn't matter.
	   jbReturn(new Boolean(false));
   }
   
 // This main program essentially emulates someone repeatedly clicking on
 // the "Clear Now..." button within Firefox until they click cancel, which
 // ends the application.
   public static void main(String[] args) {
       FirefoxPrivateData privateData = new FirefoxPrivateData();
       JComponentBreadboard clearPrivateDataForm =
          new FirefoxClearPrivateDataForm(privateData);
       while ((Boolean)
              showInputBreadboard(null, clearPrivateDataForm, "Clear Private Data")) {
          continue;
       }
   }
}
