
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;

import net.sourceforge.jcomponentbreadboard.JComponentBreadboard;

@SuppressWarnings("serial")
public class FirefoxPrintPreview extends JComponentBreadboard {


   // Simulates that part of Firefox that holds the user's printer page setup preferences.
   static class FirefoxPageSetupData implements Cloneable {
      final static int PORTRAIT = 0;
      final static int LANDSCAPE = 1;
      public int orientation = PORTRAIT; 
      public int  scale = 100; 
      public boolean shrinkToFitPageWidth = false;
      public boolean printBackground = false;
      public double  topMargin = 0.0;    
      public double  leftMargin = 0.5;
      public double  bottomMargin = 0.0;
      public double  rightMargin = 0.5;

      public String leftHeader = "--blank--";
      public String centerHeader = "--blank--";
      public String rightHeader = "--blank--";

      public String leftFooter = "--blank--";
      public String centerFooter = "--blank--";
      public String rightFooter = "--blank--";

      public String customLeftHeader = "";
      public String customCenterHeader = "";
      public String customRightHeader = "";

      public String customLeftFooter = "";
      public String customCenterFooter = "";
      public String customRightFooter = "";


      public static String[] headerChoices = {
         "--blank--", "Title", "URL", "Date/Time", "Page #", "Page # of #", "Custom..."};
         public Object clone() {
            try {
               Object result = super.clone();
               return result;
            }
            catch (CloneNotSupportedException e) {
               throw new IllegalStateException(e);
            }
         }
   }

   private FirefoxPageSetupData pageSetupData;
   final String SHRINK_TO_FIT_SCALE = "Shrink To Fit";
   final String CUSTOM_SCALE = "Custom...";
   final int N_PORTRAIT_PAGES = 9;
   final int N_PORTRAIT_LINES_PER_PAGE = 60;
   final int N_LANDSCAPE_LINES_PER_PAGE = 40;
   final int N_PORTRAIT_LINES = N_PORTRAIT_LINES_PER_PAGE*N_PORTRAIT_PAGES;
   final int N_VISIBLE_LINES = 30;
   final int N_VISIBLE_COLS = 80;
   
   private int firstVisibleLine = 0;
   private JScrollPane scrollPane;
   
   private String[] scaleDropdownList = {
         "30%", "40%", "50%", "60%", "70%", "80%", "90%",
         "100%", "125%", "150%", "175%", "200%", SHRINK_TO_FIT_SCALE, CUSTOM_SCALE};


   private boolean isOnScaleDropdownList(String value) {
      boolean result = false;
      for (int i = 0; i < scaleDropdownList.length; i++)
         if (scaleDropdownList[i].equals(value)) {
            result = true;
            break;
         }
      return result;
   }
   
   public FirefoxPrintPreview(FirefoxPageSetupData pageSetupData) {
      
      this.pageSetupData = pageSetupData;

      JButton print = (JButton) xFill(new JButton("Print..."));
      print.setMnemonic(KeyEvent.VK_P);
      JButton pageSetup = (JButton) xFill(new JButton("Page Setup..."));
      pageSetup.setMnemonic(KeyEvent.VK_U);
      JLabel pageLbl = new JLabel("Page:");
      JButton firstPage = (JButton) yFill(new JButton("|<"));
      firstPage.setMargin(new Insets(0,0,0,0));
      JButton previousPage = (JButton) yFill(new JButton(" <"));
      previousPage.setMargin(new Insets(0,0,0,0));
      JComponent page = new JTextField(6);
      JButton nextPage = (JButton) yFill(new JButton("> "));
      nextPage.setMargin(new Insets(0,0,0,0)); 
      JButton lastPage = (JButton) yFill(new JButton(">|"));
      lastPage.setMargin(new Insets(0,0,0,0));
      JLabel scaleLbl = new JLabel("Scale:");
      JComboBox scale = new JComboBox(scaleDropdownList);
      scale.setMaximumRowCount(scaleDropdownList.length);
      JRadioButton[] orientation = {
         new JRadioButton("Portrait"),
         new JRadioButton("Landscape")
      };

      orientation[0].setMnemonic(KeyEvent.VK_O);
      orientation[1].setMnemonic(KeyEvent.VK_L);

      JComponent close = xFill(new JButton("Close"));

      JTextArea simulatedPages = new JTextArea(N_VISIBLE_LINES,N_VISIBLE_COLS);
      simulatedPages.setEditable(false);
      
      scrollPane = (JScrollPane) xyFill(new JScrollPane(simulatedPages,
                   ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
                   ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS));
            
      JComponent buttonBar = xFill(new JComponentBreadboard(new Object[][] {
         {null,    NOSCALE, __,       __,       __,         BISCALE, NOSCALE, __,        __,          __,  __,      __,      BISCALE,   NOSCALE, __,     BISCALE,   NOSCALE,     __,         BISCALE,   NOSCALE},
         {NOSCALE, null, xSpace(100), null,    xSpace(100), null,    null,   null,      null,        null,null,    null,     null,      null,     null,  null,      null,        null,       null,     xSpace(100)},
         {"",      xSpace(5),  print, xSpace(5), pageSetup,xSpace(5),pageLbl,firstPage, previousPage,page,nextPage,lastPage, xSpace(5), scaleLbl, scale, xSpace(5), orientation, orientation,xSpace(5), close}   
      }));

      
      setBreadboard(new Object[][] {
         {null,    EXPANDS},   
         {NOSCALE, ySpace(5)},  
         {NOSCALE, buttonBar},
         {NOSCALE, ySpace(5)},  
         {EXPANDS, scrollPane}
      });

      jbConnect(print, "print");
      jbConnect(pageSetup, "pageSetup");
      jbConnect(firstPage, "firstPage");
      jbConnect(previousPage, "previousPage");
      jbConnect(page, "page");
      jbConnect(nextPage, "nextPage");
      jbConnect(lastPage, "lastPage");
      jbConnect(scale, "scale");
      jbConnect(orientation, "orientation");
      jbConnect(close, "close");
      jbConnect(CLOSE_BUTTON, "close");
      jbConnect(simulatedPages, "simulatedPages");
      jbConnect(scrollPane.getVerticalScrollBar(), "VScrollPosition");
      
      
   }
   // our screen shot grabbing script requires this no-arg constructor:
   public FirefoxPrintPreview() {
      this(new FirefoxPageSetupData());
   }

   // this and the next method just generate phony data so we have something to scroll,
   // and so that something changes when they switch from portrait to landscape.
   private String simPortraitLine(int portraitLine) {
      final int N_PORTRAIT_CHUNKS = 25;
      String chunk = ((portraitLine/100) % 10) + "" +
                     ((portraitLine/10) % 10) + "" +
                     (portraitLine % 10);
            
      String result = "";
      for (int i = 0; i < N_PORTRAIT_CHUNKS; i++)
         result += chunk;

      return result;
   }
   
   static String portraitString = null;
   static String landscapeString = null;
   private String getSimulatedPages(int orientation) {
      String result = "";
      if (FirefoxPageSetupData.PORTRAIT == orientation) {
         if (null == portraitString) {
            for (int i = 0; i < N_PORTRAIT_LINES; i++)
               result += simPortraitLine(i) + "\n";
            portraitString = result;
         }
         result = portraitString;
      }
      else {
         if (null == landscapeString) {
           for (int i = 0; i < N_PORTRAIT_LINES; i+=2)
              result += simPortraitLine(i) + simPortraitLine(i+1) + "\n";
           landscapeString = result;
         }
         result = landscapeString;
      }
      return result;
   }
 
   public void setPrint() {
     JOptionPane.showMessageDialog(this, "I didn't implement a Print dialog, so you will have to imagine one popping up at this point"); 
   }   

   public void setPageSetup() {
      pageSetupData = (FirefoxPageSetupData) showInputBreadboard(this,
                                          new FirefoxPageSetup(pageSetupData),
                                          "Page Setup");
   }

   public void setFirstPage() {
      firstVisibleLine = 0;
   }

   
   private int getLinesPerPage() {
      int result;
      if (FirefoxPageSetupData.PORTRAIT == pageSetupData.orientation)
         result = N_PORTRAIT_LINES_PER_PAGE;
      else
         result = N_LANDSCAPE_LINES_PER_PAGE;
      return result;
   }
   
   public void setPreviousPage() {
       firstVisibleLine = Math.max(0, firstVisibleLine - getLinesPerPage()); 
   }
   public void setNextPage() {
      firstVisibleLine = Math.min(N_PORTRAIT_LINES-getLinesPerPage(),
                                  firstVisibleLine + getLinesPerPage()); 
   }

   public String getPage() {
      int pageNum = 1 + firstVisibleLine/getLinesPerPage();
      return Integer.toString(pageNum);
   }
   public void setPage(String value) {
      int pageNum = Integer.parseInt(value);
      firstVisibleLine = Math.max(0, Math.min((pageNum-1) * getLinesPerPage(),
                                              N_PORTRAIT_LINES-getLinesPerPage()));
   }
   
   public String getPageInvalidDataMessage(String value) {
      String result = value.matches("[0-9]+") ?
                      JComponentBreadboard.DATA_IS_VALID :
                      JComponentBreadboard.REVERT_AND_BEEP;
      return result;
   }

   public void setLastPage() {
      firstVisibleLine = Math.max(0, N_PORTRAIT_LINES-getLinesPerPage());  
   }

   public String getScale() {
      String result = pageSetupData.scale + "%";
      if (pageSetupData.shrinkToFitPageWidth)
         result = SHRINK_TO_FIT_SCALE;
      else if (!isOnScaleDropdownList(result))
         result = CUSTOM_SCALE;
      
      return result;
   }
   public void setScale(String value) {
      if (value.equals(SHRINK_TO_FIT_SCALE))
         pageSetupData.shrinkToFitPageWidth = true;
      else if (value.equals(CUSTOM_SCALE)) {
         Object customScale = JOptionPane.showInputDialog(this,
"<html><b>Enter a percentage scale factor of at least 10 and at most 999:",
               "Custom...", JOptionPane.INFORMATION_MESSAGE,
               null, null, Integer.toString(pageSetupData.scale));
         if (customScale instanceof String) {
           if (((String) customScale).matches(" *[1-9][0-9] *") ||
               ((String) customScale).matches(" *[1-9][0-9][0-9] *")) {
              pageSetupData.scale = Integer.parseInt((String) customScale);
              pageSetupData.shrinkToFitPageWidth = false;
           }
           else {
              Toolkit.getDefaultToolkit().beep();
           }
         }
      }
      else {
         pageSetupData.scale = Integer.parseInt(value.substring(0,value.length()-"%".length()));
         pageSetupData.shrinkToFitPageWidth = false;
      }
   }
   // An array of radio buttons, connected and using the idiom below, is the easiest way
   // to handle radio button groups (works better than a ButtonGroup). This approach
   // can also handle groups that have unusual selection requirements (like being able
   // to deselect everything, or being able to select more than one button) without
   // much additional work.
   public boolean getOrientation(int iRow) {
      return pageSetupData.orientation == iRow;
   }
   public void setOrientation(int iRow, boolean value) {
      firstVisibleLine = 0;
      pageSetupData.orientation = iRow;
   }

   public void setClose() {
      jbReturn();
   }

   public String getSimulatedPages() {
      String result = getSimulatedPages(pageSetupData.orientation);
      return result;
   }
   public void setSimulatedPages(String value) {
      // scrollable page field isn't editable, so set does nothing
   }

   public int getVScrollPosition() {
      return (firstVisibleLine * scrollPane.getVerticalScrollBar().getMaximum())/
             N_PORTRAIT_LINES;
   }
   public void setVScrollPosition(int value) {
      firstVisibleLine = ((N_PORTRAIT_LINES+1) * value)/
                         scrollPane.getVerticalScrollBar().getMaximum();
   }

// Implements the printer page setup popup dialog (portrait/landscape, margins, etc.)   
   @SuppressWarnings("serial")
   public class FirefoxPageSetup extends JComponentBreadboard {

      final double PAGE_WIDTH = 8.5;     // in inches
      final double PAGE_HEIGHT = 11.0;
      final double PAGE_XPIXELS_PER_INCH = 12; // determined via trial and error  
      final double PAGE_YPIXELS_PER_INCH = 10; // until it looked OK on my screen.
      
      private FirefoxPageSetupData modifiedData;
      private FirefoxPageSetupData originalData;

      // applies the font used throughout this form to various objects
      private Font f = new Font("SansSerif", Font.PLAIN, 18);
      private JComponent f(JComponent jc) {
         jc.setFont(f);
         return jc;
      }
      private TitledBorder f(TitledBorder tb) {
         tb.setTitleFont(f);
         return tb;
      }
      
      public FirefoxPageSetup(FirefoxPageSetupData pageSetupData) {
         originalData = pageSetupData;
         this.modifiedData = (FirefoxPageSetupData) pageSetupData.clone();

         // beginning of panel added to the first tab on the tabbed pane:
         JComponent[] orientation = {
            f(new JRadioButton("Portrait")),
            f(new JRadioButton("Landscape")),                                  
         };

         JComponent orientationGrp = new JComponentBreadboard(new Object[][] {
            {f(new JLabel("Orientation:  ")), orientation, __},
         });

         JComponent scale = f(new JSpinner(new SpinnerNumberModel(100, 1, 200, 1)));
         JComponent shrinkToFit = f(new JCheckBox("Shrink To Fit Page Width"));

         JComponentBreadboard scaleGrp = new JComponentBreadboard(new Object[][] {
            {f(new JLabel("Scale:  ")), scale, f(new JLabel(" % ")), shrinkToFit},
         });

         JComponent formatGrp = new JComponentBreadboard(new Object[][] {
            {xySpace(10,10),null,           null},
            {null,          orientationGrp, null},
            {ySpace(10),    null,           null},
            {null,          scaleGrp,       null},
            {null,          null,           xySpace(10,10)}
         });

         formatGrp.setBorder(f(BorderFactory.createTitledBorder("Format")));

         JComponent printBackground = f(new JCheckBox("Print Background (color and images)"));

         JComponent optionsGrp = new JComponentBreadboard(new Object[][] {
            {xySpace(10,10), null,           null},
            {null,          printBackground, null},
            {null,          null,            xySpace(10,10)}
         });
         optionsGrp.setBorder(f(BorderFactory.createTitledBorder("Options")));

         JComponent formatAndOptionsForm = xFill(new JComponentBreadboard(new Object[][] {
            {xySpace(10,10), null,              null},   
            {null,           xFill(formatGrp),  null},
            {null,           xFill(optionsGrp), null},
            {null,           null,              xySpace(10,10)}
         }));

         // beginning of panel added to the second tab on the tabbed pane
         
         JComponent marginViewer = new JPanel(); // margin viewer
         
         JComponent left = xUpsize(10,f(new JSpinner(new SpinnerNumberModel(0, 0, 4, 0.1))));
         JComponent top = xUpsize(10,f(new JSpinner(new SpinnerNumberModel(0, 0, 4, 0.1))));
         JComponent right = xUpsize(10,f(new JSpinner(new SpinnerNumberModel(0, 0, 4, 0.1))));
         JComponent bottom = xUpsize(10, f(new JSpinner(new SpinnerNumberModel(0, 0, 4, 0.1))));
         JComponent leftLbl = f(new JLabel("Left: "));
         JComponent rightLbl = f(new JLabel("Right: "));
         JComponent topLbl = f(new JLabel("Top: "));
         JComponent bottomLbl = f(new JLabel("Bottom: "));

         // put them in groups so the group as a whole can be centered
         JComponent leftGrp = yAlign(0.5, new JComponentBreadboard(new Object[][] {
            {leftLbl},
            {left}
         }));
         JComponent topGrp = xAlign(0.5, new JComponentBreadboard(new Object[][] {
            {topLbl, top},
         }));
         JComponent rightGrp = yAlign(0.5, new JComponentBreadboard(new Object[][] {
            {rightLbl},
            {right}
         }));
         JComponent bottomGrp = xAlign(0.5, new JComponentBreadboard(new Object[][] {
            {bottomLbl, bottom},
         }));


         JComponent marginsGrp = new JComponentBreadboard(new Object[][] {
            {null,    BISCALE,   NOSCALE,    __,            __,         __,       BISCALE},
            {BISCALE, ySpace(1), __,         __,            __,         __,       __},
            {NOSCALE, xSpace(1), null,       null,          null,       null,     xSpace(1)}, 
            {"",      null,      null,       topGrp,         __,        null,     null}, 
            {"",      "",        leftGrp,    marginViewer,   __,        rightGrp, null},
            {"",      "",        "",         "",             "",        "",       null},
            {"",      "",        null,       bottomGrp,      __,         null,    null},
            {BISCALE, ySpace(1), __,         __,             __,         __,      __},
         });

         TitledBorder titledBorder = f(BorderFactory.createTitledBorder("Margins (inches)"));
         marginsGrp.setBorder(titledBorder);
         
         JComponent leftHeader = f(new JComboBox(FirefoxPageSetupData.headerChoices));
         JComponent centerHeader = f(new JComboBox(FirefoxPageSetupData.headerChoices));
         JComponent rightHeader = f(new JComboBox(FirefoxPageSetupData.headerChoices));
         JComponent leftFooter = f(new JComboBox(FirefoxPageSetupData.headerChoices));
         JComponent centerFooter = f(new JComboBox(FirefoxPageSetupData.headerChoices));
         JComponent rightFooter = f(new JComboBox(FirefoxPageSetupData.headerChoices));

         JComponent leftHFLbl = f(xAlign(0.5,new JLabel("Left:")));
         JComponent centerHFLbl = f(xAlign(0.5, new JLabel("Center:")));
         JComponent rightHFLbl = f(xAlign(0.5, new JLabel("Right:")));

         JComponent headerFooterGrp = new JComponentBreadboard(new Object[][] {
            {null,    EXPANDS,    __,           __         },
            {NOSCALE, leftHeader, centerHeader, rightHeader},
            {"",      leftHFLbl,  centerHFLbl,  rightHFLbl},
            {"",      leftFooter, centerFooter, rightFooter},
         });
         headerFooterGrp.setBorder(f(BorderFactory.createTitledBorder("Headers & Footers")));

         // Compute height large enough to contain marginViewer assemblage at either
         // orientation (to keep overall height rigid as orientation changes) 
         final int H = (int) (Math.max(PAGE_HEIGHT, PAGE_WIDTH) * PAGE_YPIXELS_PER_INCH) +
                       topGrp.getPreferredSize().height +
                       bottomGrp.getPreferredSize().height +
                       titledBorder.getBorderInsets(marginsGrp).top +
                       titledBorder.getBorderInsets(marginsGrp).bottom + 10;

         JComponent marginsAndHeaderFooterForm = new JComponentBreadboard(new Object[][] {
            {null,    NOSCALE,        EXPANDS,                NOSCALE},      
            {NOSCALE, xySpace(10,10), null,                   null},
            {"",      ySpace(H),      xyFill(marginsGrp),     null},
            {"",      ySpace(10),     null,                   null},
            {"",      null,           xFill(headerFooterGrp), null},
            {"",      null,           null,                   xySpace(10,10)}
         });

         // "shared" button bar beneath the tabbed pane:
         JComponent ok = f(xFill(new JButton("OK")));
         JComponent cancel = f(xFill(new JButton("Cancel")));

         JComponent buttonBar = new JComponentBreadboard(new Object[][] {
            {null,    BISCALE,    NOSCALE,     __,         __      },
            {NOSCALE, ySpace(20), xSpace(100), null,       xSpace(100)},         
            {"",      xSpace(1),  ok,          xSpace(10), cancel},
         });

         // assemble the two panels as first,second tabs of the tabbed pane
         JComponent tabbedPane = f(new JTabbedPane());
         tabbedPane.add("Format & Options", formatAndOptionsForm);
         tabbedPane.add("Margins & Header/Footer", marginsAndHeaderFooterForm);

         setBreadboard(new Object[][] {
            {tabbedPane},
            {xFill(buttonBar)}
         });

         setBorder(BorderFactory.createEmptyBorder(10,10,10,10));


         jbConnect(orientation, "orientation");
         jbConnect(scale, "scale");
         jbConnect(shrinkToFit, "shrinkToFit");
         jbConnect(printBackground, "printBackground");
         jbConnect(marginViewer, "marginViewer");
         jbConnect(left, "left");
         jbConnect(top, "top");
         jbConnect(right, "right");
         jbConnect(bottom, "bottom");
         jbConnect(leftHeader, "leftHeader");
         jbConnect(centerHeader, "centerHeader");
         jbConnect(rightHeader, "rightHeader");
         jbConnect(leftFooter, "leftFooter");
         jbConnect(centerFooter, "centerFooter");
         jbConnect(rightFooter, "rightFooter");
         jbConnect(cancel, "cancel");
         jbConnect(KeyStroke.getKeyStroke("ESCAPE"), "cancel");
         jbConnect(ok, "ok");
         jbConnect(CLOSE_BUTTON, "ok");

      }
      
      public boolean getOrientation(int iRow) {
         return (modifiedData.orientation == iRow);
      }
      
      public void setOrientation(int iRow, boolean value) {
         firstVisibleLine = 0;
         modifiedData.orientation = iRow;         
      }

      public int getScale() {
         return modifiedData.scale;
      }
      public void setScale(int value) {
         modifiedData.scale = value;
      }

      public boolean getShrinkToFit() {
         return modifiedData.shrinkToFitPageWidth;
      }
      public void setShrinkToFit(boolean value) {
         modifiedData.shrinkToFitPageWidth = value;
      }

      public boolean getPrintBackground() {return modifiedData.printBackground;}
      public void setPrintBackground(boolean value) {modifiedData.printBackground = value;}   

      private double pageWidth() {
         if (FirefoxPageSetupData.LANDSCAPE == modifiedData.orientation) 
            return PAGE_HEIGHT;
         else
            return PAGE_WIDTH;
      }
      private double pageHeight() {
         if (FirefoxPageSetupData.LANDSCAPE == modifiedData.orientation) 
            return PAGE_WIDTH;
         else
            return PAGE_HEIGHT;
      }

      public Dimension getMarginViewerPreferredSize() {
         Dimension result = new Dimension((int) (PAGE_XPIXELS_PER_INCH*pageWidth()),
                              (int) (PAGE_YPIXELS_PER_INCH*pageHeight()));
         return result;
      }
      public Border getMarginViewerBorder() {
         Border line = LineBorder.createBlackLineBorder();
         Border margin = BorderFactory.createEmptyBorder(
           (int) (PAGE_YPIXELS_PER_INCH*modifiedData.topMargin),
           (int) (PAGE_XPIXELS_PER_INCH*modifiedData.leftMargin),
           (int) (PAGE_YPIXELS_PER_INCH * modifiedData.bottomMargin),
           (int) (PAGE_XPIXELS_PER_INCH*modifiedData.rightMargin));
         Border result = BorderFactory.createCompoundBorder(margin, line);
         return result;
      }
      public Color getMarginViewerBackground() {return Color.WHITE;}

      public double getLeft() {return modifiedData.leftMargin;}
      public void setLeft(double value) {modifiedData.leftMargin=value;}

      public double getRight() {return modifiedData.rightMargin;}
      public void setRight(double value) {modifiedData.rightMargin=value;}

      public double getTop() {return modifiedData.topMargin;}
      public void setTop(double value) {modifiedData.topMargin=value;}

      public double getBottom() {return modifiedData.bottomMargin;}
      public void setBottom(double value) {modifiedData.bottomMargin=value;}

      // if the hfSelection is "Custom..." prompts user to enter new custom header/footer
      private String maybeGetCustomHeaderFooter(String hfSelection,
                                                String prevCustomHeaderFooter) {
         String result = prevCustomHeaderFooter;
         if (hfSelection.equals("Custom...")) {
            Object obj = JOptionPane.showInputDialog(this,
               "<html><h1>Enter your custom header/footer text",
               "Custom...", JOptionPane.INFORMATION_MESSAGE,
               null, null, prevCustomHeaderFooter);
            if (obj instanceof String)
               result = (String) obj;
            // else user clicked cancel, leave unchanged.
         }
         return result;
      }
      public String getLeftHeader() {return modifiedData.leftHeader;}
      public void setLeftHeader(String value) {
         modifiedData.leftHeader=value;
         modifiedData.customLeftHeader = maybeGetCustomHeaderFooter(
            value, modifiedData.customLeftHeader); 
      }
      public String getLeftHeaderToolTipText() {return "Left header";}

      public String getCenterHeader() {return modifiedData.centerHeader;}
      public void setCenterHeader(String value) {
         modifiedData.centerHeader=value;
         modifiedData.customCenterHeader = maybeGetCustomHeaderFooter(
            value, modifiedData.customCenterHeader);
      }
      public String getCenterHeaderToolTipText() {return "Center header";}

      public String getRightHeader() {return modifiedData.rightHeader;}
      public void setRightHeader(String value) {
         modifiedData.rightHeader=value;
         modifiedData.customRightHeader = maybeGetCustomHeaderFooter(
            value, modifiedData.customRightHeader);
      }
      public String getRightHeaderToolTipText() {return "Right header";}

      public String getLeftFooter() {return modifiedData.leftFooter;}
      public void setLeftFooter(String value) {
         modifiedData.leftFooter=value;
         modifiedData.customLeftFooter = maybeGetCustomHeaderFooter(
            value, modifiedData.customLeftFooter);
      }
      public String getLeftFooterToolTipText() {return "Left footer";}

      public String getCenterFooter() {return modifiedData.centerFooter;}
      public void setCenterFooter(String value) {
         modifiedData.centerFooter=value;
         modifiedData.customCenterFooter = maybeGetCustomHeaderFooter(
            value, modifiedData.customCenterFooter);
      }
      public String getCenterFooterToolTipText() {return "Center footer";}

      public String getRightFooter() {return modifiedData.rightFooter;}
      public void setRightFooter(String value) {
         modifiedData.rightFooter=value;
         modifiedData.customRightFooter = maybeGetCustomHeaderFooter(
            value, modifiedData.customRightFooter);
      }
      public String getRightFooterToolTipText() {return "Right footer";}

      public void setCancel() {
         jbReturn(originalData);  
      }
      public void setOk() {
         jbReturn(modifiedData);
      }
      public boolean getOkIsDefaultButton() {return true;}


      
   }

   public static void main(String[] args)  {
       FirefoxPageSetupData pageSetupData = new FirefoxPageSetupData();
       showMessageBreadboard(null,new FirefoxPrintPreview(pageSetupData),
                             "Print Preview"); 
   }
}
