// NumericInputDialog.java
import java.awt.Container;
import javax.swing.*;
import net.sourceforge.jcomponentbreadboard.JComponentBreadboard;

@SuppressWarnings("serial")
public class NumericInputDialog extends JComponentBreadboard {
   private double number;                   
   private double initialValue;

   public NumericInputDialog(String prompt, double initialValue) {
      this.initialValue = initialValue;
      number = initialValue;
//<<A
      JTextField numberField = new JTextField(10);
      JButton ok = new JButton("OK");
      JButton cancel = new JButton("Cancel");
      ok.setPreferredSize(cancel.getPreferredSize()); // makes OK and Cancel same size
      
      JComponentBreadboard buttonBar = new JComponentBreadboard(new Object[][] {
         {ok, xSpace(10), cancel}
      });
      buttonBar.setAlignmentX(0.5f);  // centers buttonBar horizontally
      
      setBreadboard(new Object[][] {
         {new JLabel(prompt),      numberField},
         {ySpace(10),              null},
         {buttonBar,               buttonBar}
      });

      setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
//>>A      
//<<1
      jbConnect(numberField, "number");
//>>1
//<<4
      jbConnect(cancel, "cancel");
      jbConnect(KeyStroke.getKeyStroke("ESCAPE"), "cancel");
//>>4
//<<6
      jbConnect(ok, "ok");
      jbConnect(CLOSE_BUTTON, "ok");
//>>6
      
   }
   
//<<2
   private final String NOT_A_NUMBER = "";

   public String getNumber() {
      if (Double.isNaN(number))
         return NOT_A_NUMBER;
      else
         return Double.toString(number);
   }
   public void setNumber(String value) {
      if (value.equals(NOT_A_NUMBER))
         number = Double.NaN;
      else
         number = Double.parseDouble(value);
   }
//>>2

// (optional sign) + (decimal or integer) + (optional exponent)
// as a regular expression (c.f. JDK's Pattern class javadocs)
   final String NUMBER_MATCHING_PATTERN =
       "[+-]?\\d*((\\d[.])|([.]\\d)|\\d)\\d*([Ee][+-]?\\d+)?";

//<<3
   public String getNumberInvalidDataMessage(String value) {
      String result = "<html><title>Invalid Numeric Entry</title>Your entry, \"" + value +
                      "\" is not a valid number. " +
                      "<br><br>Correct it below, or cancel to " +
                      "revert to the previous entry.<br><br> ";
// Non-popup options (uncomment one to try it):
//    result = JComponentBreadboard.REVERT_AND_BEEP;
//    result = JComponentBreadboard.REVERT_QUIETLY;
      
      if (value.equals(NOT_A_NUMBER) || 
    	    value.equals("Infinity") || value.equals("-Infinity") ||
    	    value.matches(NUMBER_MATCHING_PATTERN))
         result = JComponentBreadboard.DATA_IS_VALID;
      
      return result;
       
   }
//>>3

//<<5   
   public void setCancel() {
      jbReturn(new Double(initialValue));
   }
//>>5
//<<7
   public void setOk() {
        jbReturn(new Double(number));
   }
//>>7
//<<8
   public boolean getOkIsDefaultButton() { return true; }
//>>8   

   // Modally prompts user for a number
   public static double showNumericInputDialog(Container parent,
                                               String prompt,
                                               String title,
                                               double initialValue) {
      double result = (Double) showInputBreadboard(
         parent, new NumericInputDialog(prompt, initialValue), title);
      return result;
   }

   public static void main(String[] args) {
//<<9
      UIManager.put("Button.defaultButtonFollowsFocus", Boolean.TRUE);
//>>9
      double result = NumericInputDialog.showNumericInputDialog(
         null, "Enter a number:  ", "Enter Number", Double.NaN);
      JOptionPane.showMessageDialog(null,
                                    "Number entered = " + result);  
   }
   
}
