// RandomWalks.java
import javax.swing.*;
import net.sourceforge.jcomponentbreadboard.JComponentBreadboard;

@SuppressWarnings("serial")
public class RandomWalks extends JComponentBreadboard {
   private int[] distances = {2,2,2};

   public RandomWalks() {

      JSpinner[] distanceSpinner = new JSpinner[distances.length];
      for (int i = 0; i < distanceSpinner.length; i++) {
         distanceSpinner[i] = new JSpinner(new SpinnerNumberModel(1,1,10,1));
      }

      JButton startFirstWalk = new JButton(
         "Just take the first random walk.");
      JButton startAllWalks = new JButton(
         "Take all 3 random walks, in sequence.");
      JButton startAllWalksTwice = new JButton(
         "Take all 3 random walks. Then do it all again.");

//<<1      
      setBreadboard(new Object[][] {
         {xAlign(1, new JLabel("Walk #1 distance: ")), xFill(distanceSpinner)},
         {xAlign(1, new JLabel("Walk #2 distance: ")), ""                    },
         {xAlign(1, new JLabel("Walk #3 distance: ")), ""                    },
         {ySpace(5),                                   null                  },
         {yUpsize(10,xFill(startFirstWalk)),           __                    },
         {yUpsize(10,xFill(startAllWalks)),            __                    },
         {yUpsize(10,xFill(startAllWalksTwice)),       __                    },
      });
//>>1

      setBorder(BorderFactory.createEmptyBorder(10,10,10,10)); 

//<<2
      jbConnect(distanceSpinner, "distanceSpinner");
//>>2
      jbConnect(startFirstWalk, "startFirstWalk");
      jbConnect(startAllWalks,  "startAllWalks");
      jbConnect(startAllWalksTwice,  "startAllWalksTwice");

   }

//<<3
   public void setDistanceSpinner(int iRow, int distance) {
      distances[iRow] = distance;
   }
   public int getDistanceSpinner(int iRow) {
      return distances[iRow];
   }
//>>3

//<<6
   // take a random walk with the specified distance 
   public void takeARandomWalk(int distance) {
      double currentPosition = 0;
      
      while (Math.abs(currentPosition) < distance &&
             !getJbRunCanceled()) {
//<<4
         currentPosition += (Math.random() < 0.5) ? 1 : -1;
//>>4
         setJbRunProgress(Math.abs(currentPosition)/distance);
         try {Thread.sleep(1000);} catch (InterruptedException e) {} // wait 1 second
      }
   }
//>>6

//<<8
   // Take a sequence of random walks, each of a specified distance from the origin
   public void takeAMultiStageRandomWalk(int[] distance) {

      for (int i = 0;
           i < distance.length && !getJbRunCanceled();
           i++) {
         beginProgressMonitoredSection(
            (i+0.)/distance.length, (i+1.)/distance.length); 
           setJbRunNote("Randomly walking walk #" + (i+1));
           takeARandomWalk(distance[i]);
         endProgressMonitoredSection();  
      }
   }
//>>8

//<<9
   // take our entire sequence of 3 random walks, twice
   public void takeAMultiStageRandomWalkTwice(int[] distance) {
      final int N_SEQUENCES = 2;
      for (int i = 0;
           i < N_SEQUENCES && !getJbRunCanceled();
           i++) {
         beginProgressMonitoredSection(
            (i+0.)/N_SEQUENCES, (i+1.)/N_SEQUENCES); 
           setJbRunNote("Sequence #" + (i+1));
           takeAMultiStageRandomWalk(distance);
         endProgressMonitoredSection();  
      }
   }
//>>9

//<<5   
   public void setStartFirstWalk() {
      jbRun(new Runnable() {public void run() {
         setJbRunTitle("Walking First Walk");
         setJbRunCanBeCanceled(true);
         setJbRunNote("Randomly walking just the first walk.");
         takeARandomWalk(distances[0]);
      }});
   }
//>>5
//<<7
   public void setStartAllWalks() {
      jbRun(new Runnable() {public void run() {
         setJbRunTitle("Walking All Walks");
         setJbRunCanBeCanceled(true);
         takeAMultiStageRandomWalk(distances);
      }});
   }
//>>7
   public void setStartAllWalksTwice() {
      jbRun(new Runnable() {public void run() {
         setJbRunTitle("Walking All Walks, Twice");
         setJbRunCanBeCanceled(true);
         takeAMultiStageRandomWalkTwice(distances);
      }});
   }

   public static void main(String[] arg) {
      showMessageBreadboard(null, new RandomWalks(), "Random Walks");
   }
}
