/* Script to grab screen shots from example programs for use in user's guide
 *
 * It also generates the HTML that lists the implied interfaces of all jbConnect-able
 * JComponents.
 *
 */
import java.awt.AWTException;
import java.awt.Robot;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Date;

import javax.imageio.ImageIO;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JRadioButton;
import javax.swing.JScrollBar;
import javax.swing.JSeparator;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.SpinnerDateModel;
import javax.swing.SpinnerListModel;
import javax.swing.SpinnerNumberModel;

import net.sourceforge.jcomponentbreadboard.JComponentBreadboard;

public class ScreenShotGrabber {

   private final static String OUTPUT_FOLDER = "C:\\myDocs\\javaroot\\jcomponentbreadboard\\src\\net\\sourceforge\\jcomponentbreadboard\\doc-files\\";
   // maximum amount of time it takes for the screen to update after processing the
   // various keycodes (gives screen chance to be painted before taking the screen shot)
   private final static int MAX_SCREEN_UPDATE_TIME = 10000;
   private final static int INTRA_KEY_DELAY = 500;


   // this creates a JDialog based on the given breadboard, uses a Robot to
   // feed it the keystroke sequence, and then captures a screen shot of the
   // resulting JDialog in a JPEG which is written to the output folder 
   private static void writeScreenShot(JComponentBreadboard breadboard, String title,
                                  int[] keyCodes,
                                  String outputFilename) throws AWTException {

	 Robot r = new Robot();
     r.setAutoWaitForIdle(true);
     r.setAutoDelay(INTRA_KEY_DELAY);
     JDialog d = JComponentBreadboard.createBreadboardDialog(
        null, breadboard, title, false);
     d.setVisible(true);
     r.delay(INTRA_KEY_DELAY);  // little extra time just before startup
     for (int i = 0; i < keyCodes.length; i++) {
        if (keyCodes[i] == KeyEvent.VK_ALT ||
            keyCodes[i] == KeyEvent.VK_SHIFT ||
            keyCodes[i] == KeyEvent.VK_CONTROL) { 
          r.keyPress(keyCodes[i]);
          r.keyPress(keyCodes[i+1]);
          r.keyRelease(keyCodes[i+1]);
          r.keyRelease(keyCodes[i]);
          i++;
        }
        else {
          r.keyPress(keyCodes[i]);
          r.keyRelease(keyCodes[i]);
        }
     }
	 r.delay(MAX_SCREEN_UPDATE_TIME);
     BufferedImage i = r.createScreenCapture(d.getBounds());
     File f = new File(OUTPUT_FOLDER + outputFilename);
     try { ImageIO.write(i, "jpeg", f); }
     catch (IOException e) { e.printStackTrace(); }
     d.dispose();
   }

   @SuppressWarnings("deprecation")
public static void writeImpliedInterfaces() throws IOException {
// list of components whose implied interfaces will be tabulated:
      JComponent[] prototypes = {
         new JButton(),
         new JComponentBreadboard.JCBKeyStroke(),
         new JMenuItem(),
         new JToggleButton(),
         new JRadioButton(),
         new JCheckBox(),
         new JComboBox(),
         new JLabel(),
         new JProgressBar(),
         new JScrollBar(),
         new JSlider(),
         new JSpinner(new SpinnerNumberModel(0,0,9,1)),     // int spinner 
         new JSpinner(new SpinnerNumberModel(0.,0.,9.,1.)), // double spinner
         new JSpinner(new SpinnerDateModel()),
         new JSpinner(new SpinnerListModel()),
         new JTextField(),
         new JTextArea(),
         new JEditorPane(),
         new JPanel(),
         new JSeparator()
      };
      String impliedInterfacesAsHTML =
          JComponentBreadboard.getInjectedInterfacesTable(prototypes, "xxx");
      File f = new File(OUTPUT_FOLDER + "injected_interfaces.html");
      BufferedWriter bw = new BufferedWriter(new FileWriter(f));
      bw.write(impliedInterfacesAsHTML);
      bw.close();
   }
   
   public static void main(String args[]) throws AWTException, IOException {
       int[] keyCodes = new int[0];  // default is just display without any key presses

      writeImpliedInterfaces();
//      if (true) return;
      // sequence enters equation "2 + 2 = 4"  
      writeScreenShot(new ScrollingCalculator(),
                 "Scrolling Calculator",
                 new int[] {KeyEvent.VK_ENTER, KeyEvent.VK_ENTER, KeyEvent.VK_2, KeyEvent.VK_ENTER, KeyEvent.VK_ENTER, KeyEvent.VK_2, KeyEvent.VK_ENTER},
                 "scrollingcalculator-1.jpg");
//      if (true) return;
      writeScreenShot(new GridBagJDKExample(),
              "GridBagJDKExample",
              keyCodes,
              "gridbagjdkexample-1.jpg");
writeScreenShot(new GridBagJDKExample(),
      "GridBagJDKExample",
      new int[] {
KeyEvent.VK_ALT, KeyEvent.VK_SPACE, KeyEvent.VK_S, 
KeyEvent.VK_RIGHT,KeyEvent.VK_LEFT,KeyEvent.VK_LEFT,KeyEvent.VK_LEFT,KeyEvent.VK_LEFT,
KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,
KeyEvent.VK_ENTER},
      "gridbagjdkexample-2.jpg");
// if (true) return;
writeScreenShot(new FirefoxPrintPreview(),
                      "Print Preview",
                      new int[] {
KeyEvent.VK_TAB, KeyEvent.VK_SPACE, KeyEvent.VK_RIGHT,
KeyEvent.VK_ALT, KeyEvent.VK_SPACE, KeyEvent.VK_M, 
KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,
KeyEvent.VK_ENTER
      },
                      "firefoxprintpreview-1.jpg");
      writeScreenShot(new RandomWalks(),
              "Random Walks",
              keyCodes,
              "randomwalks-1.jpg");
    writeScreenShot(new RandomWalks(),
              "Random Walks",
 // without the up/down, the focus isn't clearly defined for some reason.
             new int[] {
//    	KeyEvent.VK_UP, KeyEvent.VK_DOWN, 
    	KeyEvent.VK_TAB, KeyEvent.VK_TAB, KeyEvent.VK_TAB, 
    	KeyEvent.VK_TAB, KeyEvent.VK_TAB, KeyEvent.VK_SPACE},
              "randomwalks-2.jpg");
     
      writeScreenShot(new CelsiusFahrenheitConverter(),
                      "C to F and F to C",
                      keyCodes,
                      "celsiusfahrenheitconverter-1.jpg"); 
      writeScreenShot(new CelsiusFahrenheitConverter(),
                      "C to F and F to C",
                      new int[] {
         KeyEvent.VK_ALT, KeyEvent.VK_SPACE, KeyEvent.VK_S,
         KeyEvent.VK_RIGHT,KeyEvent.VK_LEFT,KeyEvent.VK_LEFT,KeyEvent.VK_LEFT,
         KeyEvent.VK_DOWN,KeyEvent.VK_UP,KeyEvent.VK_UP,   
         KeyEvent.VK_ENTER
                           },        
                      "celsiusfahrenheitconverter-2.jpg"); 
      writeScreenShot(new CelsiusFahrenheitConverter(),
                      "C to F and F to C",
                      new int[] {
KeyEvent.VK_ALT, KeyEvent.VK_SPACE, KeyEvent.VK_S, 
KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,
KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,
KeyEvent.VK_ENTER,
KeyEvent.VK_1, KeyEvent.VK_PERIOD, KeyEvent.VK_PERIOD, KeyEvent.VK_3, KeyEvent.VK_ENTER,
                           },        
                      "celsiusfahrenheitconverter-3.jpg"); 
      writeScreenShot(new DateChooser(new Date()),
                      "Choose a Date",
                      keyCodes,
                      "datechooser-1.jpg");
      writeScreenShot(new FirefoxClearPrivateDataForm(),
                      "Clear Private Data",
                      keyCodes,
                      "firefoxclearprivatedataform-1.jpg");

      writeScreenShot(new JComponentBreadboard(new Object[][] {
                         {new JLabel("<html><h1>Hello, World !!!")}
                      }),
                      "HelloWorld",
                      keyCodes,
                      "helloworld-1.jpg");
      writeScreenShot(new HelloWorldPlus(),
                      "HelloWorldPlus",
                      keyCodes,
                      "helloworldplus-1.jpg");
      writeScreenShot(new HelloWorldPlus(),
              "HelloWorldPlus",
              new int[] {KeyEvent.VK_SPACE},
              "helloworldplus-2.jpg");
      writeScreenShot(new NumericInputDialog("Enter a number:  ", Double.NaN),
                "Enter Number",
                 keyCodes,        
                "numericinputdialog-1.jpg");
      // We resize the dialog so that it is as big as the error pop-up
      // is, because our routine only grabs a parent window sized chunk
      // of pixels. Then we enter an invalid number to display the
      // error message.
      writeScreenShot(new NumericInputDialog("Enter a number:  ", Double.NaN),
                "Enter Number",
                      new int[] {
KeyEvent.VK_ALT, KeyEvent.VK_SPACE, KeyEvent.VK_S, 
KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,KeyEvent.VK_RIGHT,
KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,KeyEvent.VK_DOWN,
KeyEvent.VK_ENTER,
KeyEvent.VK_1, KeyEvent.VK_PERIOD, KeyEvent.VK_PERIOD, KeyEvent.VK_3, KeyEvent.VK_ENTER,
                           },        
                "numericinputdialog-2.jpg");
   }
   
}