// ScrollingCalculator.java

import java.awt.AWTKeyStroke;
import java.awt.Font;
import java.awt.KeyboardFocusManager;
import java.awt.event.KeyEvent;
import java.util.HashSet;

import javax.swing.*;
import net.sourceforge.jcomponentbreadboard.JComponentBreadboard;

@SuppressWarnings("serial")
public class ScrollingCalculator extends JComponentBreadboard {
   final String[] operatorList = {"+", "-", "*", "/", "^", "="};
   final int MAX_ENTRIES = 1000;      
   final int VISIBLE_ENTRIES = 5;
   final String NAN_STRING = "";
   double[] entered = new double[MAX_ENTRIES];
   double[] computed = new double[MAX_ENTRIES];
   String[] operator = new String[MAX_ENTRIES];
   int firstVisibleEntry = 0;
   int maxEnteredEntry = -1;       
   int recentlyEnteredEntry = -1;  
   
   JComponent[] enteredJC  = new JTextField[VISIBLE_ENTRIES];
   JComponent[] computedJC = new JTextField[VISIBLE_ENTRIES];
   JComponent[] operatorJC = new JComboBox[VISIBLE_ENTRIES];
   JComponent scrollBar = xAlign(0,xUpsize(5, new JScrollBar()));   

   // so user can press return as well as tab to move to the next field
   private void makeEnterActLikeTab() {
	  HashSet<AWTKeyStroke> forwardKeys = new HashSet<AWTKeyStroke>();
	  forwardKeys.add(AWTKeyStroke.getAWTKeyStroke("ENTER"));
	  forwardKeys.add(AWTKeyStroke.getAWTKeyStroke("TAB"));
	  this.setFocusTraversalKeys(
			  KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS, 
			  forwardKeys);
   }
   
   public ScrollingCalculator() {
      makeEnterActLikeTab();
      Font f = new Font("Monospaced", Font.PLAIN, 16);
      for (int i = 0; i < VISIBLE_ENTRIES; i++) {
         computedJC[i] = xyFill(xAlign(0, new JTextField()));         
         computedJC[i].setFont(f);
         operatorJC[i] = new JComboBox(operatorList);
         operatorJC[i].setFont(f);
         enteredJC[i] = xyFill(new JTextField());
         enteredJC[i].setFont(f);
      }
      
      JButton resetButton = new JButton("Reset");
      resetButton.setMnemonic(KeyEvent.VK_R);
      JButton closeButton = new JButton("Close");
      closeButton.setMnemonic(KeyEvent.VK_C);

      JComponent buttonBar = yUpsize(10, xFill(new JComponentBreadboard(new Object[][] {
    	{null,    BISCALE,      NOSCALE,     NOSCALE,    NOSCALE},
    	{NOSCALE, xSpace(10),resetButton, xSpace(10), closeButton}
      })));
      
      resetCalculator();
      setBreadboard(new Object[][] {
         {null,    NOSCALE,       BISCALE,       NOSCALE  },	  	  
         {NOSCALE, ySpace(20),    computedJC[0], scrollBar}, 
         {"",      operatorJC[0], enteredJC[0],  ""       }, 
         {"",      ySpace(3),     null,          ""       },
         {"",      xSep(),        __,            ""       },
         {"",      xSep(),        __,            ""       },

         {"",      ySpace(20),    computedJC[1], ""       }, 
         {"",      operatorJC[1], enteredJC[1],  ""       }, 
         {"",      ySpace(3),     null,          ""       },
         {"",      xSep(),        __,            ""       },
         {"",      xSep(),        __,            ""       },
         
         {"",      ySpace(20),    computedJC[2], ""       }, 
         {"",      operatorJC[2], enteredJC[2],  ""       },
         {"",      ySpace(3),     null,          ""       },
         {"",      xSep(),        __,            ""       },
         {"",      xSep(),        __,            ""       },
         
         {"",      ySpace(20),    computedJC[3], ""       }, 
         {"",      operatorJC[3], enteredJC[3],  ""       }, 
         {"",      ySpace(3),     null,          ""       },
         {"",      xSep(),        __,            ""       },
         {"",      xSep(),        __,            ""       },
         
         {"",      ySpace(20),    computedJC[4], ""       }, 
         {"",      operatorJC[4], enteredJC[4],  ""       }, 
         {"",      ySpace(3),     null,          ""       },
         {"",      xSep(),        __,            ""       },
         {"",      xSep(),        __,            ""       },
         
         {"",      ySpace(10),    null,          null},
         {"",      buttonBar,     buttonBar,     null} 
      });
      setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
      jbConnect(computedJC, "computed");
      jbConnect(operatorJC, "operator");
      jbConnect(enteredJC, "entered");
      jbConnect(scrollBar, "scrollBar");
      jbConnect(resetButton, "resetButton");
      jbConnect(closeButton, "closeButton");
   }
   
   private void resetCalculator() {
      for (int i = 0; i < MAX_ENTRIES; i++) {
         entered[i] = Double.NaN;
         computed[i] = Double.NaN;
         operator[i] = "+";
      }
      computed[0] = 0;  
      firstVisibleEntry = 0;
      maxEnteredEntry = -1;
      recentlyEnteredEntry = -1;
   }
      
   private double calculateExpression(double x1, String op, double x2) {
      double result = Double.NaN;
      if (op.equals("="))  // this operator is a sort of local reset
         result = x2;      
      else if (Double.isNaN(x1) || Double.isNaN(x2))
         result = Double.NaN;
      else if (op.equals("+"))
         result = x1 + x2;
      else if (op.equals("-"))
         result = x1 - x2;
      else if (op.equals("*"))
         result = x1 * x2;
      else if (op.equals("/"))
         result = (x2 == 0) ? Double.NaN : (x1 / x2);
      else if (op.equals("^"))
         // non-integer powers of negative numbers produce a Not-a-number result
         result = (x1 < 0 && Math.rint(x2) != x2) ? Double.NaN : Math.pow(x1, x2);

      return result;      
   }
   private void recalculateEverything() {
      for (int i = 0; i <= maxEnteredEntry; i++)
         computed[i+1] = calculateExpression(computed[i], operator[i], entered[i]);
   }

   private String stringify(double x) {
      String result = NAN_STRING;
      if (!Double.isNaN(x))
         result = Double.toString(x);
      return result;     
   }
   
   public String getEntered(int iRow) {
      String result = stringify(entered[firstVisibleEntry + iRow]);
      return result;
   }
   
   public boolean getEnteredEnabled(int iRow) {
      return (firstVisibleEntry + iRow <= maxEnteredEntry+1);
   }
   // very last entry is disabled--last entry only there to show calculated result
   public boolean getEnteredEditable(int iRow) {
	   return firstVisibleEntry + iRow < MAX_ENTRIES-1;
   }
   
   public void setEntered(int iRow, String value) {
      entered[firstVisibleEntry+iRow] = value.equals(NAN_STRING)? Double.NaN : Double.parseDouble(value);
      recentlyEnteredEntry = firstVisibleEntry + iRow;
      maxEnteredEntry = Math.max(maxEnteredEntry, firstVisibleEntry + iRow);
      // implicit scrolling feature; assumes user wants to scroll down
      // after entering last visible row. Moves to operator on last visible row
      if (iRow + 1 == VISIBLE_ENTRIES &&
         firstVisibleEntry < MAX_ENTRIES-VISIBLE_ENTRIES) {
         firstVisibleEntry = firstVisibleEntry+1;    
         ((JComboBox) operatorJC[iRow]).requestFocus();
      }
      recalculateEverything();
   }
   public String getEnteredInvalidDataMessage(int iRow, String value) {
      String result = JComponentBreadboard.DATA_IS_VALID;
      try {
        @SuppressWarnings("unused") double dummy = Double.parseDouble(value);
      }
      catch (NumberFormatException e) {
         result = "Your entry, \"" + value + "\" is not a valid number, as required.\n\n" +
                  "You may correct your entry below, or click cancel to revert to the previous value.";
      }
      return result;
   }
   
   public String getComputed(int iRow) {
      String result = stringify(computed[firstVisibleEntry + iRow]);
      return result;
   }
   // does nothing because field isn't editable (must exist or exception raised)
   public void setComputed(int iRow, String value) {
   }
   public boolean getComputedEnabled(int iRow) {
      return firstVisibleEntry + iRow <= maxEnteredEntry+1;
   }
   public boolean getComputedEditable(int iRow) {
	      return false;
   }
   
   public String getOperator(int iRow) {
      return operator[firstVisibleEntry+iRow];  
   }
   public void setOperator(int iRow, String value) {
      operator[firstVisibleEntry+iRow] = value;
      if (firstVisibleEntry + iRow >= maxEnteredEntry) {
      // operator selected at the end of the list becomes the new default
         for (int i=firstVisibleEntry + iRow+1; i < MAX_ENTRIES; i++)
            operator[i] = value;
      }
      recalculateEverything();
   }
   public boolean getOperatorEnabled(int iRow) {
      return getEnteredEditable(iRow);      	   
   }
   public int getScrollBar() {return firstVisibleEntry;}
   public void setScrollBar(int value) {
        firstVisibleEntry = value;
   }
   public int getScrollBarMinimum() { return 0; }
   public int getScrollBarMaximum() {return Math.min(MAX_ENTRIES, maxEnteredEntry+2);}
   public int getScrollBarUnitIncrement() {return 1;}
   public int getScrollBarBlockIncrement() {return VISIBLE_ENTRIES-1;}
   public int getScrollBarVisibleAmount() {return VISIBLE_ENTRIES;}

   public void setResetButton() {
      resetCalculator();
   }
   public void setCloseButton() {
      jbReturn();
   }
   
   public static void main(String[] args) {
	  showMessageBreadboard(null, new ScrollingCalculator(), "Scrolling Calculator");   
   }  
}
