
import java.awt.Font;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import javax.swing.plaf.metal.*; // for the test icons

import net.sourceforge.jcomponentbreadboard.JComponentBreadboard;

/** 
 ** Tests JCComponentBreadboard's JLabel support.
 ** 
 **/
@SuppressWarnings("serial")
public class Test11JLabel extends JComponentBreadboard {
   boolean isEnabled = true;
   boolean isVisible = true;
   Icon[] metalIcons = {new MetalIconFactory.TreeControlIcon(true),
                        new MetalIconFactory.TreeFolderIcon(),
                        new MetalIconFactory.TreeLeafIcon(),
                        new MetalComboBoxIcon()
   };
   String[] iconNames ={"TreeControlIcon",
                        "TreeFolderIcon",
                        "TreeLeafIcon",
                        "ComboBoxIcon"  
   };

   int[] hOpts = {SwingConstants.LEFT, SwingConstants.CENTER, SwingConstants.RIGHT,
                 SwingConstants.LEADING, SwingConstants.TRAILING}; 
   int[] vOpts = {SwingConstants.TOP, SwingConstants.CENTER, SwingConstants.BOTTOM};

   String[] hOptNames = {"LEFT","CENTER","RIGHT","LEADING","TRAILING"};
   String[] vOptNames = {"TOP", "CENTER", "BOTTOM"};
   
   int hOpt = 0;
   int vOpt = 0;
   
   public boolean getHGrp(int iRow, int iCol) {
      return iRow == hOpt;
   }
   public void setHGrp(int iRow, int iCol, boolean isSet) {
      hOpt = iRow;
   }
   public boolean getVGrp(int iRow, int iCol) {
      return iRow == vOpt;
   }
   public void setVGrp(int iRow, int iCol, boolean isSet) {
      vOpt = iRow;
   }
   
   public boolean getLabelEnabled() {return isEnabled;}
   public boolean getLabelVisible() {return isVisible;}
   public String getLabelToolTipText() {return iconName + " is your icon.";}

   public void setEnabled(boolean isEnabled) {this.isEnabled = isEnabled;}
   public boolean getEnabled() {return isEnabled;}
   
   public void setVisible(boolean isVisible) {this.isVisible = isVisible;}
   public boolean getVisible() {return isVisible;}


   // returns the integer index (in the iconNames array) at which the given name occurs
   int nameToIndex(Object[] a, String sName) {
      int result = 0;
      for (int i = 0; i < a.length; i++)
        if (a[i].equals(sName)) result = i;
      return result;
   }
   String iconName = iconNames[0];
   public String getCombobox() {
      return iconName;
   }
   public void setCombobox(String iconName) {
      this.iconName = iconName;
   }
   public String getLabelText() {
      return " is the " + iconName + " icon.";
   }
   public Icon getLabelIcon() {
      return metalIcons[nameToIndex(iconNames, iconName)];
   }
   public Icon getLabelDisabledIcon() {
      return metalIcons[nameToIndex(iconNames, iconName)];
   }

   public int getLabelHorizontalTextPosition() {
      return hOpts[hOpt];
   }
   public int getLabelVerticalTextPosition() {
      return vOpts[vOpt];
   }
   public int getLabelIconTextGap() {
      return 25;
   }
   
   public Border getHJCBBorder() {
	  TitledBorder result = BorderFactory.createTitledBorder("Horiz.=" + hOptNames[hOpt]);
	  result.setTitleFont(new Font("SansSerif",Font.ITALIC, 20));
	  return result;
   }
   public Border getVJCBBorder() {
	  TitledBorder result = BorderFactory.createTitledBorder("Vert.=" + vOptNames[vOpt]);
	  return result;
   }   

   public Test11JLabel() {
      JLabel label = new JLabel();      
      JCheckBox enabled = new JCheckBox("Enabled (unchecking should disable component)");
      JCheckBox visible = new JCheckBox("Visible (unchecking should hide component)");
      JComboBox combobox = new JComboBox(iconNames);
      JComponent[][] hGrp = new JRadioButton[5][1]; 
      JComponent[][] vGrp = new JRadioButton[3][1];
      for (int i = 0; i < 5; i++)
         hGrp[i][0] = yAlign(0.5, new JRadioButton(hOptNames[i]));
      
      for (int i = 0; i < 3; i++)
         vGrp[i][0] = yAlign(0.5,new JRadioButton(vOptNames[i]));
      
      JComponent hJCB = new JComponentBreadboard(hGrp);      
      JComponent vJCB = new JComponentBreadboard(vGrp);
      
      JLabel msg = new JLabel(
"Verify: label, icon, tooltip change with combo. Enable, visible checkboxes work.");
      JComponent sp = ySpace(100);
      Object[][] breadboard = {
         {null,        enabled,                  visible},
         {null,        combobox,                 combobox},
         {xySpace(50, 50), null,                 null},
         {null,        hJCB,                     vJCB},
         {xSep(),      xSep(),                   xSep() },
         {sp,          xyAlign(.5,.5,label),     label},
         {xSep(),      xSep(),                   xSep() },
         {msg,         msg,                       msg}
      };
      setBreadboard(breadboard);
      jbConnect(label, "label");
      jbConnect(enabled, "enabled");
      jbConnect(visible, "visible");
      jbConnect(combobox, "combobox");
      jbConnect(hGrp, "hGrp");
      jbConnect(vGrp, "vGrp");
      jbConnect(hJCB, "hJCB");
      jbConnect(vJCB, "vJCB");
      
   }
   public static void main(String[] args) {
	   showMessageBreadboard(null, new Test11JLabel(),"Test11JLabel()");
   }
 }
